@@ -57,8 +57,11 @@ class SuicideBoard(chess.Board):
         return False
 
     def _material_balance(self):
-        return (chess.pop_count(self.occupied_co[self.turn]) -
-                chess.pop_count(self.occupied_co[not self.turn]))
+        return (chess.popcount(self.occupied_co[self.turn]) -
+                chess.popcount(self.occupied_co[not self.turn]))
+
+    def is_variant_end(self):
+        return not all(has_pieces for has_pieces in self.occupied_co)
 
     def is_variant_win(self):
         if not self.occupied_co[self.turn]:
@@ -122,11 +125,10 @@ class SuicideBoard(chess.Board):
 
             yield move
 
-    def generate_evasions(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
-        return
-        yield
+    def generate_legal_moves(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
+        if self.is_variant_end():
+            return
 
-    def generate_non_evasions(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
         # Generate captures first.
         found_capture = False
         for move in self.generate_pseudo_legal_captures():
@@ -203,6 +205,9 @@ class AtomicBoard(chess.Board):
     connected_kings = True
     one_king = True
 
+    def is_variant_end(self):
+        return not all(self.kings & side for side in self.occupied_co)
+
     def is_variant_win(self):
         return self.kings and not self.kings & self.occupied_co[not self.turn]
 
@@ -216,12 +221,12 @@ class AtomicBoard(chess.Board):
         if self.pawns or self.queens:
             return False
 
-        if chess.pop_count(self.KNIGHT | self.BISHOP | self.ROOK) == 1:
+        if chess.popcount(self.KNIGHT | self.BISHOP | self.ROOK) == 1:
             return True
 
         # Only knights.
         if self.occupied == (self.kings | self.knights):
-            return chess.pop_count(self.knights) <= 2
+            return chess.popcount(self.knights) <= 2
 
         # Only bishops.
         if self.occupied == (self.kings | self.bishops):
@@ -237,23 +242,15 @@ class AtomicBoard(chess.Board):
         # Can castle onto attacked squares if they are connected to the
         # enemy king.
         enemy_kings = self.kings & self.occupied_co[not self.turn]
-        enemy_king = chess.bit_scan(enemy_kings)
-        while enemy_king != -1 and enemy_king is not None:
+        for enemy_king in chess.scan_forward(enemy_kings):
             path &= ~chess.BB_KING_ATTACKS[enemy_king]
-            enemy_king = chess.bit_scan(enemy_kings, enemy_king + 1)
 
         return super(AtomicBoard, self)._attacked_for_king(path)
 
     def _kings_connected(self):
-        kings = self.kings & self.occupied_co[chess.WHITE]
-        king_square = chess.bit_scan(kings)
-        while king_square != -1 and king_square is not None:
-            if chess.BB_KING_ATTACKS[king_square] & self.kings & self.occupied_co[chess.BLACK]:
-                return True
-
-            king_square = chess.bit_scan(kings, king_square + 1)
-
-        return False
+        white_kings = self.kings & self.occupied_co[chess.WHITE]
+        black_kings = self.kings & self.occupied_co[chess.BLACK]
+        return any(chess.BB_KING_ATTACKS[sq] & black_kings for sq in chess.scan_forward(white_kings))
 
     def _push_capture(self, move, capture_square, piece_type, was_promoted):
         # Explode the capturing piece.
@@ -261,10 +258,8 @@ class AtomicBoard(chess.Board):
 
         # Explode all non pawns around.
         explosion_radius = chess.BB_KING_ATTACKS[move.to_square] & ~self.pawns
-        explosion = chess.bit_scan(explosion_radius)
-        while explosion != -1 and explosion is not None:
+        for explosion in chess.scan_forward(explosion_radius):
             self._remove_piece_at(explosion)
-            explosion = chess.bit_scan(explosion_radius, explosion + 1)
 
         # Destroy castling rights.
         self.castling_rights &= ~explosion_radius
@@ -282,6 +277,9 @@ class AtomicBoard(chess.Board):
         return was_into_check
 
     def is_legal(self, move):
+        if self.is_variant_end():
+            return False
+
         if not self.is_pseudo_legal(move):
             return False
 
@@ -299,8 +297,6 @@ class AtomicBoard(chess.Board):
             if self.is_legal(move):
                 yield move
 
-    generate_evasions = generate_non_evasions = generate_legal_moves
-
     def status(self):
         status = super(SuicideBoard, self).status()
         status &= ~chess.STATUS_OPPOSITE_CHECK
@@ -321,6 +317,9 @@ class KingOfTheHillBoard(chess.Board):
     tbw_suffix = tbz_suffix = None
     tbw_magic = tbz_magic = None
 
+    def is_variant_end(self):
+        return self.kings & BB_HILL
+
     def is_variant_win(self):
         return self.kings & self.occupied_co[self.turn] & BB_HILL
 
@@ -355,38 +354,39 @@ class RacingKingsBoard(chess.Board):
     def is_legal(self, move):
         return super(RacingKingsBoard, self).is_legal(move) and not self._gives_check(move)
 
-    def generate_evasions(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
-        for move in super(RacingKingsBoard, self).generate_evasions(from_mask, to_mask):
+    def generate_legal_moves(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
+        for move in super(RacingKingsBoard, self).generate_legal_moves(from_mask, to_mask):
             if not self._gives_check(move):
                 yield move
 
-    def generate_non_evasions(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
-        for move in super(RacingKingsBoard, self).generate_non_evasions(from_mask, to_mask):
-            if not self._gives_check(move):
-                yield move
+    def is_variant_end(self):
+        if not self.kings & chess.BB_RANK_8:
+            return False
+
+        if self.turn == chess.WHITE or self.kings & self.occupied_co[chess.BLACK] & chess.BB_RANK_8:
+            return True
+
+        black_kings = self.kings & self.occupied_co[chess.BLACK]
+        if not black_kings:
+            return True
+
+        black_king = chess.msb(black_kings)
+
+        # White has reached the backrank. The game is over if black can not
+        # also reach the backrank on the next move. Check if there are any
+        # safe squares for the king.
+        targets = chess.BB_KING_ATTACKS[black_king] & chess.BB_RANK_8
+        return all(self.attackers_mask(chess.WHITE, target) for target in chess.scan_forward(targets))
 
     def is_variant_draw(self):
         in_goal = self.kings & chess.BB_RANK_8
-        return in_goal & self.occupied_co[chess.WHITE] and in_goal & self.occupied_co[chess.BLACK]
+        return all(in_goal & side for side in self.occupied_co)
 
     def is_variant_loss(self):
-        if self.is_variant_draw():
-            return False
-
-        if self.turn == chess.WHITE:
-            return self.kings & self.occupied_co[chess.BLACK] & chess.BB_RANK_8
-        else:
-            if not self.kings & self.occupied_co[chess.WHITE] & chess.BB_RANK_8:
-                return False
-
-            # Black can not reach the backrank on the next move.
-            return not any(move for move in self.generate_legal_moves(self.kings, chess.BB_RANK_8))
+        return self.is_variant_end() and not self.kings & self.occupied_co[self.turn] & chess.BB_RANK_8
 
     def is_variant_win(self):
-        if self.is_variant_draw():
-            return False
-
-        return self.kings & self.occupied_co[self.turn] & chess.BB_RANK_8
+        return self.is_variant_end() and self.kings & self.occupied_co[self.turn] & chess.BB_RANK_8
 
     def is_insufficient_material(self):
         return False
@@ -394,7 +394,20 @@ class RacingKingsBoard(chess.Board):
     def status(self):
         status = super(RacingKingsBoard, self).status()
         if self.is_check():
-            status |= chess.STATUS_ILLEGAL_CHECK
+            status |= chess.STATUS_RACE_CHECK
+        if self.turn == chess.BLACK and all(self.occupied_co[co] & self.kings & chess.BB_RANK_8 for co in chess.COLORS):
+            status |= chess.STATUS_RACE_OVER
+        if self.pawns:
+            status |= chess.STATUS_RACE_MATERIAL
+        for color in chess.COLORS:
+            if chess.popcount(self.occupied_co[color] & self.knights) > 2:
+                status |= chess.STATUS_RACE_MATERIAL
+            if chess.popcount(self.occupied_co[color] & self.bishops) > 2:
+                status |= chess.STATUS_RACE_MATERIAL
+            if chess.popcount(self.occupied_co[color] & self.rooks) > 2:
+                status |= chess.STATUS_RACE_MATERIAL
+            if chess.popcount(self.occupied_co[color] & self.queens) > 1:
+                status |= chess.STATUS_RACE_MATERIAL
         return status
 
 
@@ -413,6 +426,9 @@ class HordeBoard(chess.Board):
     def reset(self):
         self.set_fen(type(self).starting_fen)
 
+    def is_variant_end(self):
+        return not all(has_pieces for has_pieces in self.occupied_co)
+
     def is_variant_draw(self):
         return not self.occupied
 
@@ -429,7 +445,7 @@ class HordeBoard(chess.Board):
         status = super(HordeBoard, self).status()
         status &= ~chess.STATUS_NO_WHITE_KING
 
-        if chess.pop_count(self.occupied_co[chess.WHITE]) <= 36:
+        if chess.popcount(self.occupied_co[chess.WHITE]) <= 36:
             status &= ~chess.STATUS_TOO_MANY_WHITE_PIECES
             status &= ~chess.STATUS_TOO_MANY_WHITE_PAWNS
 
@@ -555,6 +571,9 @@ class ThreeCheckBoard(chess.Board):
             epd.append(self._epd_operations(operations))
         return " ".join(epd)
 
+    def is_variant_end(self):
+        return any(remaining_checks <= 0 for remaining_checks in self.remaining_checks)
+
     def is_variant_draw(self):
         return self.remaining_checks[chess.WHITE] <= 0 and self.remaining_checks[chess.BLACK] <= 0
 
@@ -660,35 +679,18 @@ class CrazyhouseBoard(chess.Board):
 
     def legal_drop_squares_mask(self):
         king_bb = self.kings & self.occupied_co[self.turn]
-        king_square = chess.bit_scan(king_bb)
-        if king_square is None or king_square == -1:
+        if not king_bb:
             return ~self.occupied
 
+        king_square = chess.msb(king_bb)
         king_attackers = self.attackers_mask(not self.turn, king_square)
-        num_attackers = chess.pop_count(king_attackers)
 
-        if num_attackers == 0:
+        if not king_attackers:
             return ~self.occupied
-        elif num_attackers == 1:
-            king_rank_mask = chess.RANK_MASK[king_bb]
-            king_file_mask = chess.FILE_MASK[king_bb]
-            king_diag_ne = chess.DIAG_MASK_NE[king_bb]
-            king_diag_nw = chess.DIAG_MASK_NW[king_bb]
-
-            if king_rank_mask == chess.RANK_MASK[king_attackers]:
-                rank_pieces = king_rank_mask & self.occupied
-                return chess.RANK_ATTACKS[king_bb][rank_pieces] & ~self.occupied & chess.RANK_ATTACKS[king_attackers][rank_pieces]
-            elif king_file_mask == chess.FILE_MASK[king_attackers]:
-                file_pieces = king_file_mask & self.occupied
-                return chess.FILE_ATTACKS[king_bb][file_pieces] & ~self.occupied & chess.FILE_ATTACKS[king_attackers][file_pieces]
-            elif king_diag_ne == chess.DIAG_MASK_NE[king_attackers]:
-                ne_pieces = king_diag_ne & self.occupied
-                return chess.DIAG_ATTACKS_NE[king_bb][ne_pieces] & ~self.occupied & chess.DIAG_ATTACKS_NE[king_attackers][ne_pieces]
-            elif king_diag_nw == chess.DIAG_MASK_NW[king_attackers]:
-                nw_pieces = king_diag_nw & self.occupied
-                return chess.DIAG_ATTACKS_NW[king_bb][nw_pieces] & ~self.occupied & chess.DIAG_ATTACKS_NW[king_attackers][nw_pieces]
-
-        return chess.BB_VOID
+        elif chess.popcount(king_attackers) == 1:
+            return chess.BB_BETWEEN[king_square][chess.msb(king_attackers)] & ~self.occupied
+        else:
+            return chess.BB_VOID
 
     def legal_drop_squares(self):
         return chess.SquareSet(self.legal_drop_squares_mask())
@@ -715,25 +717,17 @@ class CrazyhouseBoard(chess.Board):
             return super(CrazyhouseBoard, self).is_legal(move)
 
     def generate_pseudo_legal_drops(self, to_mask=chess.BB_ALL):
-        to_mask = to_mask & ~self.occupied
-        to_square = chess.bit_scan(to_mask)
-        while to_square != -1 and to_square is not None:
+        for to_square in chess.scan_forward(to_mask & ~self.occupied):
             for pt, count in self.pockets[self.turn].pieces.items():
                 if count and (pt != chess.PAWN or not chess.BB_BACKRANKS & chess.BB_SQUARES[to_square]):
                     yield chess.Move(to_square, to_square, drop=pt)
-            to_square = chess.bit_scan(to_mask, to_square + 1)
 
     def generate_legal_drops(self, to_mask=chess.BB_ALL):
         return self.generate_pseudo_legal_drops(to_mask=self.legal_drop_squares_mask() & to_mask)
 
-    def generate_non_evasions(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
+    def generate_legal_moves(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
         return itertools.chain(
-            super(CrazyhouseBoard, self).generate_non_evasions(from_mask, to_mask),
-            self.generate_legal_drops(from_mask & to_mask))
-
-    def generate_evasions(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
-        return itertools.chain(
-            super(CrazyhouseBoard, self).generate_evasions(from_mask, to_mask),
+            super(CrazyhouseBoard, self).generate_legal_moves(from_mask, to_mask),
             self.generate_legal_drops(from_mask & to_mask))
 
     # TODO: zobrist_hash
@@ -797,11 +791,11 @@ class CrazyhouseBoard(chess.Board):
     def status(self):
         status = super(CrazyhouseBoard, self).status()
 
-        if chess.pop_count(self.pawns) + self.pockets[chess.WHITE].count(chess.PAWN) + self.pockets[chess.BLACK].count(chess.PAWN) <= 16:
+        if chess.popcount(self.pawns) + self.pockets[chess.WHITE].count(chess.PAWN) + self.pockets[chess.BLACK].count(chess.PAWN) <= 16:
             status &= ~chess.STATUS_TOO_MANY_BLACK_PAWNS
             status &= ~chess.STATUS_TOO_MANY_WHITE_PAWNS
 
-        if chess.pop_count(self.occupied) + len(self.pockets[chess.WHITE]) + len(self.pockets[chess.BLACK]) <= 32:
+        if chess.popcount(self.occupied) + len(self.pockets[chess.WHITE]) + len(self.pockets[chess.BLACK]) <= 32:
             status &= ~chess.STATUS_TOO_MANY_BLACK_PIECES
             status &= ~chess.STATUS_TOO_MANY_WHITE_PIECES
 
